--
-- IncreasedUnloadingTime
--
-- Author: coolfarmer
-- Description: Main class for IncreasedUnloadingTime.
-- Name: IncreasedUnloadingTime
--
-- Copyright (c) coolfarmer, 2021

---@class IncreasedUnloadingTime
IncreasedUnloadingTime = {}

-- Defining how the unloading will be slower, 500 means that the unloading time will slow down by 500% (5 times slower)
-- 
IncreasedUnloadingTime.AUGERWAGON_BACK_NODE = 280
IncreasedUnloadingTime.AUGERWAGON_PIPE_NODE = 200
IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE = 450
IncreasedUnloadingTime.AUGERWAGON_BLOW_PIPE_NODE = 400

IncreasedUnloadingTime.TRAILER_BACK_NODE = 150
IncreasedUnloadingTime.TRAILER_SIDE_NODE = 120
IncreasedUnloadingTime.TRAILER_GRAIN_DOOR_NODE = 500

IncreasedUnloadingTime.SEMITRAILER_BACK_NODE = 150
IncreasedUnloadingTime.SEMITRAILER_UNDER_NODE = 250
IncreasedUnloadingTime.SEMITRAILER_GRAIN_DOOR_NODE = 400
IncreasedUnloadingTime.SEMITRAILER_SLIDING_NODE = 250

IncreasedUnloadingTime.FORAGEWAGON_BACK_NODE = 170;
IncreasedUnloadingTime.FORAGEWAGON_SIDE_NODE = 150;

IncreasedUnloadingTime.COMBINE_PIPE_NODE = 115;
IncreasedUnloadingTime.VINEHARVESTER_BACK_NODE = 115;
IncreasedUnloadingTime.MIXERWAGON_SIDE_NODE = 250;
IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE = 320;
IncreasedUnloadingTime.WATERTRAILER_ALL_NODE = 200;
IncreasedUnloadingTime.STRAWBLOWER_BLOW_NODE = 250;


--Adding some code to the function loadDischargeNode to transform the emptySpeed value
function IncreasedUnloadingTime.inj_loadDischargeNode(self, superFunc, xmlFile, key, entry)
    superFunc(self, xmlFile, key, entry)
	
	if entry.node == nil then
        return false
    end
	
	local dischargeNodeName = getName(entry.node)
	
	local vehicleType = self.typeName
	
	--local storeItem = g_storeManager:getItemByXMLFilename(self.configFileName)
	--print("START ----------------------------------------------------------------------------------------")
	--print("Name: " ..storeItem.name.. " (type: " ..vehicleType.. ")")  -- Pacesetter (type: trainTrailer)
	--print("Vehicle XML path: " ..self.configFileName)  -- data/vehicles/rudolphSohn/tdk301RP/tdk301RP.xml
	--print("Current node idx: " ..key) --self.dischargeable.dischargeNode(0) 
	--print("Current node: " ..dischargeNodeName) -- dischargeNodeGrainDoor
	--print("Original emptySpeed: " ..entry.emptySpeed)
	
	local incompatibleTypes = Set { "bigBag", "pallet" }
	if incompatibleTypes[vehicleType] then
	    return false
	end
	
	local jointType = nil
	if (self.spec_attachable ~= nil) then
		for _, inputAttacherJoint in pairs(self.spec_attachable.inputAttacherJoints) do
			jointType = inputAttacherJoint.jointType
		end
	end
	
	-- Semitrailer
	if (vehicleType == "trailer" and jointType == AttacherJoints.JOINTTYPE_SEMITRAILER) then
	
		if (dischargeNodeName == "dischargeNodeBack") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.SEMITRAILER_UNDER_NODE / 100)
			
		-- Discharge from back (another name)
		elseif (dischargeNodeName == "dischargeNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.SEMITRAILER_BACK_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeMiddle") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.SEMITRAILER_UNDER_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeFront") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.SEMITRAILER_UNDER_NODE / 100)
			
		elseif (dischargeNodeName == "dischargeNodeGrainDoor") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.SEMITRAILER_GRAIN_DOOR_NODE / 100)
		
		else
		
		end
	
	-- Trailer
	--
	-- Defining how the unloading will be slower for each node
	-- 500 means that the unloading time will slow down by 500% (5 times slower)
	elseif (vehicleType == "trailer") then
	
		if (dischargeNodeName == "dischargeNodeBack") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.TRAILER_BACK_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeLeft") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.TRAILER_SIDE_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeRight") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.TRAILER_SIDE_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeGrainDoor") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.TRAILER_GRAIN_DOOR_NODE / 100)
		
		-- Discharge from back (another name)
		elseif (dischargeNodeName == "dischargeNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.TRAILER_BACK_NODE / 100)
		
		else
		
		end
	
	-- Combine
	elseif (vehicleType == "combineDrivable") then
		entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.COMBINE_PIPE_NODE / 100)
	
	-- Grape/Olive Harvester
	elseif (vehicleType == "vineHarvester") then
		entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.VINEHARVESTER_BACK_NODE / 100)
	
	-- Auger wagon
	elseif (vehicleType == "augerWagon") then
	
		-- Standard auger wagon
		if (dischargeNodeName == "pipeRaycastNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_PIPE_NODE / 100)
		
		-- Ex.: Convey-All CST 1550
		elseif (dischargeNodeName == "pipeRaycastNode1") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE / 100)
		
		elseif (dischargeNodeName == "pipeRaycastNode2") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE / 100)
			
		elseif (dischargeNodeName == "pipeRaycastNode3") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE / 100)
			
		elseif (dischargeNodeName == "pipeRaycastNode4") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE / 100)
			
		elseif (dischargeNodeName == "pipeRaycastNode5") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE / 100)
			
		elseif (dischargeNodeName == "pipeRaycastNode6") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_SMALL_PIPE_NODE / 100)
		
		-- Ex.: SUW 5000 (Discharge back)
		elseif (dischargeNodeName == "dischargeNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_BACK_NODE / 100)
		
		-- Ex.: SUW 5000 (Discharge pipe)
		elseif (dischargeNodeName == "dischargeNodePipe") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.AUGERWAGON_BLOW_PIPE_NODE / 100)
			
		else
			
		end
	
	-- Mixer Wagon
	elseif (vehicleType == "mixerWagon") then
	
		if (dischargeNodeName == "dischargeNodeLeft") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.MIXERWAGON_SIDE_NODE / 100)
			
		elseif (dischargeNodeName == "dischargeNodeRight") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.MIXERWAGON_SIDE_NODE / 100)
			
		else
			
		end
	
	-- Drivable Mixer Wagon
	elseif (vehicleType == "drivableMixerWagon") then
	
		if (dischargeNodeName == "dischargeNodeLeft") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE / 100)
			
		elseif (dischargeNodeName == "dischargeNodeRight") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE / 100)
			
		elseif (dischargeNodeName == "dischargeFrontLeft") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeFrontRight") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE / 100)
			
		elseif (dischargeNodeName == "dischargeBackLeft") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE / 100)
			
		elseif (dischargeNodeName == "dischargeBackRight") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.DRIVABLEMIXERWAGON_ALL_NODE / 100)
			
		else
			
		end
	
	-- Water Trailer
	elseif (vehicleType == "waterTrailer") then
	
		if (dischargeNodeName == "dischargeNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.WATERTRAILER_ALL_NODE / 100)
		
		else
			
		end
	
	-- Straw Blower
	elseif (vehicleType == "strawBlower") then
	
		if (dischargeNodeName == "dischargeNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.STRAWBLOWER_BLOW_NODE / 100)
		
		else
			
		end
	
	-- Forage Wagon
	elseif (vehicleType == "forageWagon") then
	
		if (dischargeNodeName == "dischargeNode") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.FORAGEWAGON_BACK_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeLeft") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.FORAGEWAGON_SIDE_NODE / 100)
		
		elseif (dischargeNodeName == "dischargeNodeRight") then
			entry.emptySpeed = entry.emptySpeed / (IncreasedUnloadingTime.FORAGEWAGON_SIDE_NODE / 100)
		
		else
			
		end
		
	else
		-- nothing to do
	end
	
	--print("Modified emptySpeed: " ..entry.emptySpeed)
	--print("END   ----------------------------------------------------------------------------------------")
	--print(" ")
	
	return true
end

---Early hook into adding vehicle specializations
function IncreasedUnloadingTime.installSpecializations(vehicleTypeManager, specializationManager, modDirectory, modName)
    Dischargeable.loadDischargeNode = Utils.overwrittenFunction(Dischargeable.loadDischargeNode, IncreasedUnloadingTime.inj_loadDischargeNode)
end

-- help functions
function Set (list)
  local set = {}
  for _, l in ipairs(list) do set[l] = true end
  return set
end